"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
/**
 * Koa Server Side Events Middleware
 *
 * @class KoaSSE
 * @extends {Transform}
 * @implements {IKoaSSE}
 */
class KoaSSE extends stream_1.Transform {
    /**
     * Creates an instance of KoaSSE.
     *
     * @param {Context} ctx
     * @param {IKoaSSEOptions} options
     */
    constructor(ctx, options) {
        super({
            writableObjectMode: true
        });
        this.options = options;
        ctx.req.socket.setTimeout(0);
        ctx.req.socket.setNoDelay(true);
        ctx.req.socket.setKeepAlive(true);
        ctx.set("Content-Type", "text/event-stream");
        ctx.set("Cache-Control", "no-cache, no-transform");
        ctx.set("Connection", "keep-alive");
        this.keepAlive();
    }
    /**
     * Send "heartbeat" comment to keep connection alive
     *
     */
    keepAlive() {
        this.push(":\n\n");
    }
    /**
     * Send server side event
     *
     * @param {(IKoaSSEvent | string)} data
     */
    send(data) {
        try {
            this.write(data);
        }
        catch (error) {
            console.error("Cannot write to already destroyed stream");
        }
    }
    /**
     * Send close event source message and end stream
     *
     */
    close() {
        const data = {
            event: this.options.closeEvent
        };
        this.end(data);
    }
    /**
     * Transform stream base class method
     *
     * @param {*} data
     * @param {string} _
     * @param {TransformCallback} __
     * @returns {void}
     */
    _transform(data, _, cb) {
        // Handle string data 
        if (typeof data === "string") {
            this.push(`data: ${data}\n\n`);
            return cb();
        }
        // Handle object data 
        if (data.id) {
            this.push(`id: ${data.id}\n`);
        }
        if (data.event) {
            this.push(`event: ${data.event}\n`);
        }
        const text = typeof data.data === "object"
            ? JSON.stringify(data.data)
            : data.data;
        this.push(`data: ${text}\n\n`);
        return cb();
    }
}
exports.default = KoaSSE;
